@@ -91,6 +91,92 @@ sub SetRecipients {
         push @To, $ticket->Requestors->MemberEmailAddresses;
     }
 
+    # custom role syntax:   gives:
+    #   name                  (undef,    role name,  Cc)
+    #   RT::CustomRole-#      (role id,  undef,      Cc)
+    #   name/To               (undef,    role name,  To)
+    #   RT::CustomRole-#/To   (role id,  undef,      To)
+    #   name/Cc               (undef,    role name,  Cc)
+    #   RT::CustomRole-#/Cc   (role id,  undef,      Cc)
+    #   name/Bcc              (undef,    role name,  Bcc)
+    #   RT::CustomRole-#/Bcc  (role id,  undef,      Bcc)
+
+    # this has to happen early because adding To addresses affects how Cc
+    # is handled
+
+    my $custom_role_re = qr!
+                           ( # $1 match everything for error reporting
+
+                           # word boundary
+                           \b
+
+                           # then RT::CustomRole-# or a role name
+                           (?:
+                               RT::CustomRole-(\d+)    # $2 role id
+                             | ( \w+ )                 # $3 role name
+                           )
+
+                           # then, optionally, a type after a slash
+                           (?:
+                               /
+                               (To | Cc | Bcc)         # $4 type
+                           )?
+
+                           # finally another word boundary, either from
+                           # the end of role identifier or from the end of type
+                           \b
+                           )
+                         !x;
+    while ($arg =~ m/$custom_role_re/g) {
+        my ($argument, $role_id, $name, $type) = ($1, $2, $3, $4);
+        my $role;
+
+        if ($name) {
+            # skip anything that is a core Notify argument
+            next if $name eq 'All'
+                 || $name eq 'Owner'
+                 || $name eq 'Requestor'
+                 || $name eq 'AdminCc'
+                 || $name eq 'Cc'
+                 || $name eq 'OtherRecipients'
+                 || $name eq 'AlwaysNotifyActor';
+
+            my $roles = RT::CustomRoles->new( $self->CurrentUser );
+            $roles->Limit( FIELD => 'Name', VALUE => $name, CASESENSITIVE => 0 );
+
+            # custom roles are named uniquely, but just in case there are
+            # multiple matches, bail out as we don't know which one to use
+            $role = $roles->First;
+            if ( $role ) {
+                $role = undef if $roles->Next;
+            }
+        }
+        else {
+            $role = RT::CustomRole->new( $self->CurrentUser );
+            $role->Load( $role_id );
+        }
+
+        unless ($role && $role->id) {
+            $RT::Logger->debug("Unable to load custom role from scrip action argument '$argument'");
+            next;
+        }
+
+        my @role_members = (
+            $ticket->RoleGroup($role->GroupType)->MemberEmailAddresses,
+            $ticket->QueueObj->RoleGroup($role->GroupType)->MemberEmailAddresses,
+        );
+
+        if (!$type || $type eq 'Cc') {
+            push @Cc, @role_members;
+        }
+        elsif ($type eq 'Bcc') {
+            push @Bcc, @role_members;
+        }
+        elsif ($type eq 'To') {
+            push @To, @role_members;
+        }
+    }
+
     if ( $arg =~ /\bCc\b/ ) {
 
         #If we have a To, make the Ccs, Ccs, otherwise, promote them to To
