@@ -182,6 +182,35 @@ sub LimitNotEmpty {
     return;
 }
 
+=head2 LimitHasFilename
+
+Limit result set to attachments with not empty filename.
+
+=cut
+
+sub LimitHasFilename {
+    my $self = shift;
+
+    $self->Limit(
+        ENTRYAGGREGATOR => 'AND',
+        FIELD           => 'Filename',
+        OPERATOR        => 'IS NOT',
+        VALUE           => 'NULL',
+        QUOTEVALUE      => 0,
+    );
+
+    if ( RT->Config->Get('DatabaseType') ne 'Oracle' ) {
+        $self->Limit(
+            ENTRYAGGREGATOR => 'AND',
+            FIELD           => 'Filename',
+            OPERATOR        => '!=',
+            VALUE           => '',
+        );
+    }
+
+    return;
+}
+
 =head2 LimitByTicket $ticket_id
 
 Limit result set to attachments of a ticket.
