@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2014 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -284,7 +284,9 @@ sub SetContent {
             return(0, "Content couldn't be frozen");
         }
     }
-    return $self->_Set( Field => 'Content', Value => $content );
+    my ($ok, $msg) = $self->_Set( Field => 'Content', Value => $content );
+    return ($ok, $self->loc("Attribute updated")) if $ok;
+    return ($ok, $msg);
 }
 
 =head2 SubValue KEY
@@ -311,12 +313,9 @@ Deletes the subvalue with the key NAME
 sub DeleteSubValue {
     my $self = shift;
     my $key = shift;
-    my %values = $self->Content();
-    delete $values{$key};
-    $self->SetContent(%values);
-
-    
-
+    my $values = $self->Content();
+    delete $values->{$key};
+    $self->SetContent($values);
 }
 
 
