@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2014 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -61,6 +61,10 @@ our @GROUPINGS = (
 
     Queue  => 'Queue',                  #loc_left_pair
 
+    InitialPriority => 'Priority',          #loc_left_pair
+    FinalPriority   => 'Priority',          #loc_left_pair
+    Priority        => 'Priority',          #loc_left_pair
+
     Owner         => 'User',            #loc_left_pair
     Creator       => 'User',            #loc_left_pair
     LastUpdatedBy => 'User',            #loc_left_pair
@@ -94,6 +98,9 @@ our %GROUPINGS_META = (
         },
         Localize => 1,
     },
+    Priority => {
+        Sort => 'numeric raw',
+    },
     User => {
         SubFields => [grep RT::User->_Accessible($_, "public"), qw(
             Name RealName NickName
@@ -283,10 +290,8 @@ our %STATISTICS_META = (
             my $self = shift;
             my $field = shift || 'id';
 
-            # UseSQLForACLChecks may add late joins
-            my $joined = ($self->_isJoined || RT->Config->Get('UseSQLForACLChecks')) ? 1 : 0;
             return (
-                FUNCTION => ($joined ? 'DISTINCT COUNT' : 'COUNT'),
+                FUNCTION => 'COUNT',
                 FIELD    => 'id'
             );
         },
@@ -464,6 +469,33 @@ sub SetupGroupings {
 
     $self->FromSQL( $args{'Query'} ) if $args{'Query'};
 
+    # Apply ACL checks
+    $self->CurrentUserCanSee if RT->Config->Get('UseSQLForACLChecks');
+
+    # See if our query is distinct
+    if (not $self->{'joins_are_distinct'} and $self->_isJoined) {
+        # If it isn't, we need to do this in two stages -- first, find
+        # the distinct matching tickets (with no group by), then search
+        # within the matching tickets grouped by what is wanted.
+        my @match = (0);
+        $self->Columns( 'id' );
+        while (my $row = $self->Next) {
+            push @match, $row->id;
+        }
+
+        # Replace the query with one that matches precisely those
+        # tickets, with no joins.  We then mark it as having been ACL'd,
+        # since it was by dint of being in the search results above
+        $self->CleanSlate;
+        while ( @match > 1000 ) {
+            my @batch = splice( @match, 0, 1000 );
+            $self->Limit( FIELD => 'Id', OPERATOR => 'IN', VALUE => \@batch );
+        }
+        $self->Limit( FIELD => 'Id', OPERATOR => 'IN', VALUE => \@match );
+        $self->{'_sql_current_user_can_see_applied'} = 1
+    }
+
+
     %GROUPINGS = @GROUPINGS unless keys %GROUPINGS;
 
     my $i = 0;
@@ -641,27 +673,24 @@ sub SortEntries {
     for ( my $i = 0; $i < @groups; $i++ ) {
         my $group_by = $groups[$i];
         my $idx = $i+1;
-        my $method;
 
-        # If this is a CF, traverse the values being used for labels.
-        # If they all look like numbers or undef, flag for a numeric sort
-
-        my $looks_like_number;
-        if ( $group_by->{'KEY'} eq 'CF' ){
-            $looks_like_number = 1;
+        my $order = $group_by->{'META'}{Sort} || 'label';
+        my $method = $order =~ /label$/ ? 'LabelValue' : 'RawValue';
 
+        unless ($order =~ /^numeric/) {
+            # Traverse the values being used for labels.
+            # If they all look like numbers or undef, flag for a numeric sort.
+            my $looks_like_number = 1;
             foreach my $item (@data){
-                my $cf_label = $item->[0]->RawValue($group_by->{'NAME'});
+                my $label = $item->[0]->$method($group_by->{'NAME'});
 
                 $looks_like_number = 0
-                    unless (not defined $cf_label)
-                    or Scalar::Util::looks_like_number( $cf_label );
+                    unless (not defined $label)
+                    or Scalar::Util::looks_like_number( $label );
             }
+            $order = "numeric $order" if $looks_like_number;
         }
 
-        my $order = $looks_like_number ? 'numeric label' : 'label';
-        $order = $group_by->{'META'}{Sort} if exists $group_by->{'META'}{Sort};
-
         if ( $order eq 'label' ) {
             push @SORT_OPS, sub { $_[0][$idx] cmp $_[1][$idx] };
             $method = 'LabelValue';
@@ -677,7 +706,7 @@ sub SortEntries {
             $method = 'LabelValue';
         }
         elsif ( $order eq 'raw' ) {
-            push @SORT_OPS, sub { $_[0][$idx] cmp $_[1][$idx] };
+            push @SORT_OPS, sub { ($_[0][$idx]//'') cmp ($_[1][$idx]//'') };
             $method = 'RawValue';
         }
         elsif ( $order eq 'numeric raw' ) {
