@@ -131,14 +131,6 @@ sub Load  {
         ($ok, $msg) = $self->LoadByCol('Name', $identifier);
     }
 
-    if (@_) {
-        RT->Deprecated(
-            Arguments => "Template as second argument",
-            Remove    => "4.4",
-        );
-        $self->{'Template'} = shift;
-    }
-
     return wantarray ? ($ok, $msg) : $ok;
 }
 
@@ -167,10 +159,8 @@ sub LoadAction  {
         $self->{'TemplateObj'} = $args{'TemplateObj'};
     }
 
-    $self->ExecModule =~ /^(\w+)$/;
-    my $module = $1;
-    my $type = "RT::Action::". $module;
-
+    my $module = $self->ExecModule;
+    my $type = 'RT::Action::' . $module;
     $type->require or die "Require of $type action module failed.\n$@\n";
 
     return $self->{'Action'} = $type->new(
@@ -181,42 +171,6 @@ sub LoadAction  {
     );
 }
 
-
-=head2 TemplateObj
-
-Return this action's template object.  Deprecated.
-
-=cut
-
-sub TemplateObj {
-    my $self = shift;
-    RT->Deprecated(
-        Remove => "4.4",
-    );
-
-    if ( !$self->{'TemplateObj'} ) {
-        return undef unless $self->{Template};
-        $self->{'TemplateObj'} = RT::Template->new( $self->CurrentUser );
-        $self->{'TemplateObj'}->Load( $self->{'Template'} );
-
-        if ( ( $self->{'TemplateObj'}->__Value('Queue') == 0 )
-            && $self->{'_TicketObj'} ) {
-            my $tmptemplate = RT::Template->new( $self->CurrentUser );
-            my ( $ok, $err ) = $tmptemplate->LoadQueueTemplate(
-                Queue => $self->{'_TicketObj'}->QueueObj->id,
-                Name  => $self->{'TemplateObj'}->Name);
-
-            if ( $tmptemplate->id ) {
-                # found the queue-specific template with the same name
-                $self->{'TemplateObj'} = $tmptemplate;
-            }
-        }
-
-    }
-
-    return ( $self->{'TemplateObj'} );
-}
-
 sub Prepare  {
     my $self = shift;
     $self->{_Message_ID} = 0;
