@@ -72,13 +72,11 @@ package RT::ScripCondition;
 use strict;
 use warnings;
 
-
 use base 'RT::Record';
 
 
 sub Table {'ScripConditions'}
 
-
 sub _Accessible  {
     my $self = shift;
     my %Cols = ( Name  => 'read',
@@ -157,21 +155,19 @@ sub LoadCondition  {
                  TicketObj => undef,
                  @_ );
 
-    #TODO: Put this in an eval
-    $self->ExecModule =~ /^(\w+)$/;
-    my $module = $1;
-    my $type = "RT::Condition::". $module;
-
+    my $module = $self->ExecModule;
+    my $type = 'RT::Condition::' . $module;
     $type->require or die "Require of $type condition module failed.\n$@\n";
 
-    $self->{'Condition'}  = $type->new ( 'ScripConditionObj' => $self,
-                                         'TicketObj' => $args{'TicketObj'},
-                                         'ScripObj' => $args{'ScripObj'},
-                                         'TransactionObj' => $args{'TransactionObj'},
-                                         'Argument' => $self->Argument,
-                                         'ApplicableTransTypes' => $self->ApplicableTransTypes,
-                                         CurrentUser => $self->CurrentUser
-                                       );
+    return $self->{'Condition'} = $type->new(
+        ScripConditionObj => $self,
+        TicketObj => $args{'TicketObj'},
+        ScripObj => $args{'ScripObj'},
+        TransactionObj => $args{'TransactionObj'},
+        Argument => $self->Argument,
+        ApplicableTransTypes => $self->ApplicableTransTypes,
+        CurrentUser => $self->CurrentUser
+    );
 }
 
 
