@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2014 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -177,8 +177,7 @@ sub JoinTargetToAdded {
         ALIAS      => $alias,
         FIELD      => 'ObjectId',
         OPERATOR   => 'IN',
-        QUOTEVALUE => 0,
-        VALUE      => "(". join( ',', map $dbh->quote($_), @ids ) .")",
+        VALUE      => [ @ids ],
     );
 
     return $alias;
