@@ -64,6 +64,7 @@ use warnings;
 
 package RT::Squish::JS;
 use base 'RT::Squish';
+use JavaScript::Minifier::XS ();
 
 =head2 Squish
 
@@ -91,38 +92,8 @@ sub Squish {
 }
 
 sub Filter {
-    my $self    = shift;
-    my $content = shift;
-
-    my $minified;
-    my $jsmin = RT->Config->Get('JSMinPath');
-    if ( $jsmin && -x $jsmin ) {
-        my $input = $content;
-        my ( $output, $error );
-
-        # If we're running under fastcgi, STDOUT and STDERR are tied
-        # filehandles, which cause IPC::Run3 to flip out.  Construct
-        # temporary, not-tied replacements for it to see instead.
-        my $stdout = IO::Handle->new;
-        $stdout->fdopen( 1, 'w' );
-        local *STDOUT = $stdout;
-        my $stderr = IO::Handle->new;
-        $stderr->fdopen( 2, 'w' );
-        local *STDERR = $stderr;
-
-        local $SIG{'CHLD'} = 'DEFAULT';
-        require IPC::Run3;
-        IPC::Run3::run3( [$jsmin], \$input, \$output, \$error );
-        if ( $? >> 8 ) {
-            $RT::Logger->warning("failed to jsmin: $error ");
-        }
-        else {
-            $content  = $output;
-            $minified = 1;
-        }
-    }
-
-    return $content;
+    my ( $self, $content ) = @_;
+    return JavaScript::Minifier::XS::minify($content);
 }
 
 1;
