@@ -78,6 +78,7 @@ use Text::Template;
 use MIME::Entity;
 use MIME::Parser;
 use Scalar::Util 'blessed';
+use RT::Interface::Web;
 
 sub _Accessible {
     my $self = shift;
@@ -500,6 +501,18 @@ sub _ParseContent {
         $args{'loc'} = sub { $self->loc(@_) };
     }
 
+    $args{'EscapeURI'} = sub {
+        my $str = shift;
+        RT::Interface::Web::EscapeURI( \$str );
+        return $str;
+    };
+
+    $args{'EscapeHTML'} = sub {
+        my $str = shift;
+        RT::Interface::Web::EscapeHTML( \$str );
+        return $str;
+    };
+
     if ($self->Type eq 'Perl') {
         return $self->_ParseContentPerl(
             Content      => $content,
