@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2014 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -125,37 +125,19 @@ sub LimitToQueue {
 }
 
 
-=head2 Next
+=head2 AddRecord
 
-Returns the next template that this user can see.
+Overrides the collection to ensure that only templates the user can see
+are returned.
 
 =cut
 
-sub Next {
+sub AddRecord {
     my $self = shift;
+    my ($record) = @_;
 
-
-    my $templ = $self->SUPER::Next();
-    if ((defined($templ)) and (ref($templ))) {
-
-        # If it's part of a queue, and the user can read templates in
-        # that queue, or the user can globally read templates, show it
-        if ($templ->Queue && $templ->CurrentUserHasQueueRight('ShowTemplate') or
-            $templ->CurrentUser->HasRight(Object => $RT::System, Right => 'ShowTemplate') or
-            $templ->CurrentUser->HasRight(Object => $RT::System, Right => 'ShowGlobalTemplates')) {
-            return($templ);
-        }
-
-        #If the user doesn't have the right to show this template
-        else {
-            return($self->Next());
-        }
-    }
-    #if there never was any template
-    else {
-        return(undef);
-    }
-
+    return unless $record->CurrentUserCanRead;
+    return $self->SUPER::AddRecord( $record );
 }
 
 RT::Base->_ImportOverlays();
