@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2014 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -130,27 +130,12 @@ sub LimitToTicket {
 }
 
 
-sub Next {
+sub AddRecord {
     my $self = shift;
+    my ($record) = @_;
 
-    my $Transaction = $self->SUPER::Next();
-    if ((defined($Transaction)) and (ref($Transaction))) {
-        # If the user can see the transaction's type, then they can
-        #  see the transaction and we should hand it back.
-        if ($Transaction->Type) {
-            return($Transaction);
-        }
-
-        #If the user doesn't have the right to show this ticket
-        else {
-            return($self->Next());
-        }
-    }
-
-    #if there never was any ticket
-    else {
-        return(undef);
-    }
+    return unless $record->CurrentUserCanSee;
+    return $self->SUPER::AddRecord($record);
 }
 
 RT::Base->_ImportOverlays();
